@@ -202,23 +202,38 @@ class AmendMode(Command):
 
 class ApplyDiffSelection(Command):
 
-    def __init__(self, staged, offset, selection_text, apply_to_worktree):
+    def __init__(self, first_line_idx, last_line_idx, has_selection,
+                 reverse, apply_to_worktree):
         Command.__init__(self)
-        self.staged = staged
-        self.offset = offset
-        self.selection_text = selection_text
+        self.first_line_idx = first_line_idx
+        self.last_line_idx = last_line_idx
+        self.has_selection = has_selection
+        self.reverse = reverse
         self.apply_to_worktree = apply_to_worktree
 
     def do(self):
-        # The normal worktree vs index scenario
-        parser = DiffParser(self.model,
-                            filename=self.model.filename,
-                            cached=self.staged,
-                            reverse=self.apply_to_worktree)
-        status, out, err = \
-        parser.process_diff_selection(self.offset,
-                                      self.selection_text,
-                                      apply_to_worktree=self.apply_to_worktree)
+        parser = DiffParser(self.model.filename, self.model.diff_text)
+        if self.has_selection:
+            patch = parser.generate_patch(self.first_line_idx,
+                                          self.last_line_idx,
+                                          reverse=self.reverse)
+        else:
+            patch = parser.generate_hunk_patch(self.first_line_idx,
+                                               reverse=self.reverse)
+        if patch is None:
+            return
+
+        cfg = gitcfg.current()
+        tmp_path = utils.tmp_filename('patch')
+        try:
+            core.write(tmp_path, patch,
+                       encoding=cfg.file_encoding(self.model.filename))
+            if self.apply_to_worktree:
+                status, out, err = self.model.apply_diff_to_worktree(tmp_path)
+            else:
+                status, out, err = self.model.apply_diff(tmp_path)
+        finally:
+            os.unlink(tmp_path)
         Interaction.log_status(status, out, err)
         self.model.update_file_status(update_index=True)
 
@@ -356,14 +371,16 @@ class Commit(ResetMode):
         # Create the commit message file
         msg = self.strip_comments(self.msg)
         tmpfile = utils.tmp_filename('commit-message')
-        core.write(tmpfile, msg)
+        try:
+            core.write(tmpfile, msg)
 
-        # Run 'git commit'
-        status, out, err = self.model.git.commit(F=tmpfile,
-                                                 v=True,
-                                                 gpg_sign=self.sign,
-                                                 amend=self.amend)
-        core.unlink(tmpfile)
+            # Run 'git commit'
+            status, out, err = self.model.git.commit(F=tmpfile,
+                                                     v=True,
+                                                     gpg_sign=self.sign,
+                                                     amend=self.amend)
+        finally:
+            core.unlink(tmpfile)
 
         if status == 0:
             ResetMode.do(self)
@@ -646,19 +663,17 @@ class DeleteRemoteBranch(Command):
 class Diff(Command):
     """Perform a diff and set the model's current text."""
 
-    def __init__(self, filenames, cached=False):
+    def __init__(self, filename, cached=False, deleted=False):
         Command.__init__(self)
-        # Guard against the list of files being empty
-        if not filenames:
-            return
         opts = {}
         if cached:
             opts['ref'] = self.model.head
-        self.new_filename = filenames[0]
-        self.old_filename = self.model.filename
+        self.new_filename = filename
         self.new_mode = self.model.mode_worktree
-        self.new_diff_text = gitcmds.diff_helper(filename=self.new_filename,
-                                                 cached=cached, **opts)
+        self.new_diff_text = gitcmds.diff_helper(filename=filename,
+                                                 cached=cached,
+                                                 deleted=deleted,
+                                                 **opts)
 
 
 class Diffstat(Command):
@@ -681,8 +696,8 @@ class Diffstat(Command):
 class DiffStaged(Diff):
     """Perform a staged diff on a file."""
 
-    def __init__(self, filenames):
-        Diff.__init__(self, filenames, cached=True)
+    def __init__(self, filename, deleted=None):
+        Diff.__init__(self, filename, cached=True, deleted=deleted)
         self.new_mode = self.model.mode_index
 
 
@@ -1127,7 +1142,7 @@ class Rescan(Command):
 class Refresh(Command):
     """Update refs and refresh the index"""
 
-    SHORTCUT = 'Ctrl+R'
+    SHORTCUTS = ('Ctrl+R', 'F5')
 
     @staticmethod
     def name():
@@ -1309,19 +1324,18 @@ class SetDiffText(Command):
 class ShowUntracked(Command):
     """Show an untracked file."""
 
-    def __init__(self, filenames):
+    def __init__(self, filename):
         Command.__init__(self)
-        self.filenames = filenames
+        self.new_filename = filename
         self.new_mode = self.model.mode_untracked
-        self.new_diff_text = ''
-        if filenames:
-            self.new_diff_text = self.diff_text_for(filenames[0])
+        self.new_diff_text = self.diff_text_for(filename)
 
     def diff_text_for(self, filename):
         cfg = gitcfg.current()
         size = cfg.get('cola.readsize', 1024 * 2)
         try:
-            result = core.read(filename, size=size)
+            result = core.read(filename, size=size,
+                               encoding='utf-8', errors='ignore')
         except:
             result = ''
 
@@ -1468,21 +1482,21 @@ class Tag(Command):
         log_msg = (N_('Tagging "%(revision)s" as "%(name)s"') %
                    dict(revision=self._revision, name=self._name))
         opts = {}
-        if self._message:
-            opts['F'] = utils.tmp_filename('tag-message')
-            core.write(opts['F'], self._message)
+        try:
+            if self._message:
+                opts['F'] = utils.tmp_filename('tag-message')
+                core.write(opts['F'], self._message)
 
-        if self._sign:
-            log_msg += ' (%s)' % N_('GPG-signed')
-            opts['s'] = True
+            if self._sign:
+                log_msg += ' (%s)' % N_('GPG-signed')
+                opts['s'] = True
+            else:
+                opts['a'] = bool(self._message)
             status, output, err = self.model.git.tag(self._name,
                                                      self._revision, **opts)
-        else:
-            opts['a'] = bool(self._message)
-            status, output, err = self.model.git.tag(self._name,
-                                                     self._revision, **opts)
-        if 'F' in opts:
-            os.unlink(opts['F'])
+        finally:
+            if 'F' in opts:
+                os.unlink(opts['F'])
 
         if output:
             log_msg += '\n' + (N_('Output: %s') % output)
