@@ -3,6 +3,7 @@
 """
 from __future__ import division, absolute_import, unicode_literals
 
+import mimetypes
 import os
 import re
 import subprocess
@@ -24,6 +25,26 @@ from cola.widgets import defs
 from cola.compat import ustr
 
 
+KNOWN_FILE_MIME_TYPES = [
+    ('text',    'script.png'),
+    ('image',   'image.png'),
+    ('python',  'script.png'),
+    ('ruby',    'script.png'),
+    ('shell',   'script.png'),
+    ('perl',    'script.png'),
+    ('octet',   'binary.png'),
+]
+
+KNOWN_FILE_EXTENSIONS = {
+    '.java':    'script.png',
+    '.groovy':  'script.png',
+    '.cpp':     'script.png',
+    '.c':       'script.png',
+    '.h':       'script.png',
+    '.cxx':     'script.png',
+}
+
+
 def connect_action(action, fn):
     action.connect(action, SIGNAL('triggered()'), fn)
 
@@ -142,10 +163,10 @@ class TreeWidgetItem(QtGui.QTreeWidgetItem):
 
     TYPE = QtGui.QStandardItem.UserType + 101
 
-    def __init__(self, path, icon, exists):
+    def __init__(self, path, icon, deleted):
         QtGui.QTreeWidgetItem.__init__(self)
         self.path = path
-        self.exists = exists
+        self.deleted = deleted
         self.setIcon(0, cached_icon_from_path(icon))
         self.setText(0, path)
 
@@ -153,11 +174,6 @@ class TreeWidgetItem(QtGui.QTreeWidgetItem):
         return self.TYPE
 
 
-def create_treewidget_item(text, filename, exists=True):
-    """Creates a QTreeWidgetItem with text and the icon at filename."""
-    return TreeWidgetItem(text, filename, exists)
-
-
 def paths_from_indexes(model, indexes,
                        item_type=TreeWidgetItem.TYPE,
                        item_filter=None):
@@ -420,49 +436,38 @@ def set_items(widget, items):
     add_items(widget, items)
 
 
-def icon_file(filename, staged=False, untracked=False):
-    """Returns a file path representing a corresponding file path."""
-    exists = True
-    if staged:
-        exists = core.exists(filename)
-        if exists:
-            ifile = resources.icon('staged-item.png')
-        else:
-            ifile = resources.icon('removed.png')
+def icon_name_for_filename(filename):
+    """Returns an icon name based on the filename."""
+    mimetype = mimetypes.guess_type(filename)[0]
+    if mimetype is not None:
+        mimetype = mimetype.lower()
+        for filetype, icon_name in KNOWN_FILE_MIME_TYPES:
+            if filetype in mimetype:
+                return icon_name
+    extension = os.path.splitext(filename)[1]
+    return KNOWN_FILE_EXTENSIONS.get(extension.lower(), 'generic.png')
+
+
+def icon_from_filename(filename):
+    icon_name = icon_name_for_filename(filename)
+    return cached_icon_from_path(resources.icon(icon_name))
+
+
+def create_treeitem(filename, staged=False, deleted=False, untracked=False):
+    """Given a filename, return a TreeListItem suitable for adding to a
+    QListWidget.  "staged", "deleted, and "untracked" control whether to use
+    the appropriate icons."""
+    if deleted:
+        icon_name = 'removed.png'
+    elif staged:
+        icon_name = 'staged-item.png'
     elif untracked:
-        ifile = resources.icon('untracked.png')
+        icon_name = 'untracked.png'
     else:
-        (ifile, exists) = utils.file_icon(filename)
-    return (ifile, exists)
+        icon_name = icon_name_for_filename(filename)
+    return TreeWidgetItem(filename, resources.icon(icon_name), deleted=deleted)
 
 
-def icon_for_file(filename, staged=False, untracked=False):
-    """Returns a QIcon for a particular file path."""
-    ifile = icon_file(filename, staged=staged, untracked=untracked)
-    return icon(ifile)
-
-
-def create_treeitem(filename, staged=False, untracked=False, check=True):
-    """Given a filename, return a QListWidgetItem suitable
-    for adding to a QListWidget.  "staged" and "untracked"
-    controls whether to use the appropriate icons."""
-    if check:
-        (ifile, exists) = icon_file(filename,
-                                    staged=staged, untracked=untracked)
-    else:
-        exists = True
-        ifile = resources.icon('staged.png')
-    return create_treewidget_item(filename, ifile, exists=exists)
-
-
-def update_file_icons(widget, items, staged=True,
-                      untracked=False, offset=0):
-    """Populate a QListWidget with custom icon items."""
-    for idx, model_item in enumerate(items):
-        item = widget.item(idx+offset)
-        if item:
-            item.setIcon(icon_for_file(model_item, staged, untracked))
-
 @memoize
 def cached_icon(key):
     """Maintain a cache of standard icons and return cache entries."""
@@ -584,6 +589,13 @@ def center_on_screen(widget):
     widget.move(cx - widget.width()//2, cy - widget.height()//2)
 
 
+def default_size(parent, width, height):
+    """Return the parent's size, or the provided defaults"""
+    if parent is not None:
+        width = parent.width()
+        height = parent.height()
+    return (width, height)
+
 @memoize
 def theme_icon(name):
     """Grab an icon from the current theme with a fallback
@@ -785,7 +797,7 @@ def mimedata_from_paths(paths):
     return mimedata
 
 
-def mimetypes():
+def path_mimetypes():
     return ['text/uri-list', 'text/x-moz-url']
 
 # Syntax highlighting
